#ifndef __ThreadPartitioner_h__
#define __ThreadPartitioner_h__

#include "BModule.h"
#include "CorrelationExaminer.h"

static const float CORRELATION_THRESHOLD = 0.65;

class ThreadPartitioner
{
public:
    static Vector<BModule> partition( Vector<ThreadStateGraph *> threads )
    {
        return ThreadPartitioner( threads ).run();
    }

private:
    Vector<ThreadStateGraph *> _threads;
    Vector<BModule> _modules;
    Vector<Event> _allEvents;
    CorrelationExaminer _correlationExaminer;

    ThreadPartitioner( Vector<ThreadStateGraph *> threads ) : _threads( threads ), _allEvents( _threads[0]->_events )
    {
    }

    void dumpModules()
    {
        printf( "Number of modules: %u\n", _modules.size() );

        for ( unsigned i = 0; i < _modules.size(); ++i )
        {
            printf( "Module %u: ", i + 1 );

            for ( unsigned j = 0; j < _modules[i]._threads.size(); ++j )
            {
                printf( "%s ", _modules[i]._threads[j]->_threadName.ascii() );
            }

            printf( "\n" );
        }

        fflush( 0 );
    }

    Vector<BModule> run()
    {
        initModules();

        // printf( "Initial dump:\n" );
        // dumpModules();

        for ( unsigned i = 0; i < _allEvents.size(); ++i )
        {
            // printf( "\n\nWorking on event: %u\n", _allEvents[i].type() );
            joinModulesByEvent( _allEvents[i] );
            // dumpModules();
        }

        return _modules;
    }

    void initModules()
    {
        for ( unsigned i = 0; i < _threads.size(); ++i )
        {
            BModule module;
            module.add( _threads[i] );
            _modules.append( module );
        }
    }

    void joinModulesByEvent( Event event )
    {
        BModule joinedModule;

        Vector<BModule>::iterator it = _modules.begin();

        while ( it != _modules.end() )
        {
            if ( highCorrelation( *it, event ) )
            {
                combineModules( *it, joinedModule );
                _modules.erase( it );
            }
            else
            {
                ++it;
            }
        }

        if ( !joinedModule.empty() )
        {
            _modules.append( joinedModule );
        }
    }

    bool highCorrelation( BModule &module, Event &event )
    {
        for ( unsigned i = 0; i < module._threads.size(); ++i )
        {
            if ( _correlationExaminer.findCorrelation( *module._threads[i], event ) > CORRELATION_THRESHOLD )
            {
                return true;
            }
        }

        return false;
    }

    static void combineModules( BModule &toAdd, BModule &combined )
    {
        for ( unsigned i = 0; i < toAdd._threads.size(); ++i )
        {
            combined.add( toAdd._threads[i] );
        }
    }
};

#endif // __ThreadPartitioner_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
