#ifndef __CustomizedOracle_h__
#define __CustomizedOracle_h__

#include "Event.h"
#include "IEvent.h"
#include "IOracle.h"

class CustomizedOracle : public IOracle::OracleChooser
{
public:
    Event choose( Vector<Event> &enabledEvents )
    {
        if ( idleEventEnabled( enabledEvents ) && otherOptionsExist( enabledEvents ) )
        {
            return nonIdleEvent( enabledEvents );
        }

        return firstEvent( enabledEvents );
    }

private:
    static bool idleEventEnabled( Vector<Event> &enabledEvents )
    {
        return enabledEvents.exists( Event( IEvent::IDLE_EVENT ) );
    }

    static bool otherOptionsExist( Vector<Event> &enabledEvents )
    {
        return enabledEvents.size() > 1;
    }

    static Event nonIdleEvent( Vector<Event> &enabledEvents )
    {
        for ( unsigned i = 0; i < enabledEvents.size(); ++i )
        {
            if ( enabledEvents[i].type() != IEvent::IDLE_EVENT )
            {
                return enabledEvents[i];
            }
        }

        throw Error( Error::ORACLE_ERROR );
    }

    static Event firstEvent( Vector<Event> &enabledEvents )
    {
        return enabledEvents[0];
    }


};

#endif // __CustomizedOracle_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
