#ifndef __HttpErrorHandler_h__
#define __HttpErrorHandler_h__

#include "BThread.h"
#include "File.h"
#include "IEvent.h"

class HttpErrorHandler : public BThread
{
public:
    void entryPoint()
    {
        while ( true )
        {
            waitForErrorRequest();
            sendResponse();
            endSession();
            doneHandlingRequest();
        }
    }

private:
    enum {
        MAX_FILE_SIZE = 4096,
    };

    void waitForErrorRequest()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_GENERATE_ERROR );
        bSync( _requested, _watched, _blocked, "Wait" );
    }

    void sendResponse()
    {
        String errorString( "HTTP/1.1 404 Not Found\r\n"
                            "Server: BP Server\r\n"
                            "Cache-Control: no-cache,no-store\r\n"
                            "Content-Type: text/html; charset=\%s\r\n"
                            "Connection: close\r\n"
                            "\r\n"
                            "<HTML>\n"
                            "<HEAD><TITLE>404 Not Found</TITLE></HEAD>\n"
                            "<BODY>\n"
                            "<H4>404 Not Found</H4>\n"
                            "File not found. Hopefully, this isn't a BP error...\n"
                            "</BODY>\n"
                            "</HTML>\n" );

        HeapData errorMessage = ConstSimpleData( errorString.ascii(), errorString.length() );

        clearEventVectors();
        _requested.append( Event( IEvent::DATA_FROM_HTTP_TO_TCP, errorMessage ) );
        _blocked.append( IEvent::HTTP_GENERATE_ERROR );

        bSync( _requested, _watched, _blocked, "Send" );
    }

    void endSession()
    {
        clearEventVectors();

        _requested.append( IEvent::HTTP_END_SESSION );
        _blocked.append( IEvent::HTTP_GENERATE_ERROR );
        bSync( _requested, _watched, _blocked, "EndSession" );
    }

    void doneHandlingRequest()
    {
        clearEventVectors();

        _requested.append( IEvent::HTTP_DONE_HANDLING_REQUEST );
        _blocked.append( IEvent::HTTP_GENERATE_ERROR );
        bSync( _requested, _watched, _blocked, "Done" );
    }
};

#endif // __HttpErrorHandler_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
