#ifndef __HttpGetSimulator_h__
#define __HttpGetSimulator_h__

#include "BThread.h"
#include "File.h"
#include "IEvent.h"
#include "ServerConfiguration.h"

class HttpGetSimulator : public BThread
{
public:
    void entryPoint()
    {
        while ( true )
        {
            waitForHttpGet();
            requestAllAlternatives();

            if ( sentData() )
            {
                sendFileData();
                endSession();
                doneHandlingRequest();
            }
        }
    }

private:
    void waitForHttpGet()
    {
        clearEventVectors();

        _watched.append( Event( IEvent::HTTP_GET_REQUEST ) );
        bSync( _requested, _watched, _blocked, "Wait" );
    }

    void requestAllAlternatives()
    {
        clearEventVectors();

        _requested.append( IEvent::HTTP_GENERATE_ERROR );
        _requested.append( IEvent::HTTP_GENERATE_FOUND );
        _requested.append( IEvent::DATA_FROM_HTTP_TO_TCP );

        _blocked.append( IEvent::HTTP_GET_REQUEST );

        bSync( _requested, _watched, _blocked, "AllAlternatives" );
    }

    bool sentData()
    {
        return lastEvent().type() == IEvent::DATA_FROM_HTTP_TO_TCP;
    }

    void sendFileData()
    {
        clearEventVectors();

        _requested.append( IEvent::DATA_FROM_HTTP_TO_TCP );
        _blocked.append( IEvent::HTTP_GET_REQUEST );

        bSync( _requested, _watched, _blocked, "Chunk1" );
        bSync( _requested, _watched, _blocked, "Chunk2" );
        bSync( _requested, _watched, _blocked, "Chunk3" );
    }

    void endSession()
    {
        clearEventVectors();
        _requested.append( IEvent::HTTP_END_SESSION );
        _blocked.append( IEvent::HTTP_GET_REQUEST );
        bSync( _requested, _watched, _blocked, "EndSession" );
    }

    void doneHandlingRequest()
    {
        clearEventVectors();
        _requested.append( IEvent::HTTP_DONE_HANDLING_REQUEST );
        _blocked.append( IEvent::HTTP_GET_REQUEST );
        bSync( _requested, _watched, _blocked, "Done" );
    }
};

#endif // __HttpGetSimulator_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
