#ifndef __IEvent_h__
#define __IEvent_h__

#include "Event.h"
#include "Stringf.h"

class IEvent
{
public:
    enum EventType {
        HTTP_DATA_RECEIVED = 0,
        HTTP_DONE_HANDLING_REQUEST = 1,
        HTTP_END_SESSION = 2,
        HTTP_GENERATE_ERROR = 3,
        HTTP_GENERATE_FOUND = 4,
        HTTP_GET_REQUEST = 5,

        TCP_DATA_RECEIVED = 6,
        TCP_DONE_HADNLING_ACK = 7,
        TCP_DONE_HANDLING_DATA = 8,
        TCP_FIN_REQUEST = 9,
        TCP_PUSH_REQUEST = 10,
        TCP_SEND_ACK = 11,
        TCP_SEND_DATA = 12,
        TCP_SEND_PUSH = 13,
        TCP_SYN_REQUEST = 14,

        TCP_ANNOUNCE_TARGET_PORT = 15,

        SYN_HANDLER_REQUEST_SEQUENCE_NUMBER = 16,
        SEQUENCE_NUMBER_FOR_SYN_HANDLER = 17,

        FIN_HANDLER_REQUEST_SEQUENCE_NUMBER = 18,
        SEQUENCE_NUMBER_FOR_FIN_HANDLER = 19,

        ACK_SENDER_REQUEST_SEQUENCE_NUMBER = 20,
        SEQUENCE_NUMBER_FOR_ACK_SENDER = 21,

        PUSH_SENDER_REQUEST_SEQUENCE_NUMBER = 22,
        SEQUENCE_NUMBER_FOR_PUSH_SENDER = 23,

        DATA_FROM_TCP_TO_HTTP = 24,
        DATA_FROM_HTTP_TO_TCP = 25,

        NUMBER_OF_EVENTS = 26,
    };

    static Vector<Event> allEvents()
    {
        Vector<Event> result;

        for ( unsigned i = 0; i < NUMBER_OF_EVENTS; ++i )
        {
            result.append( Event( i ) );
        }

        return result;
    }

    static Vector<Event> allEventsExcept( IEvent::EventType eventType )
    {
        Vector<Event> result;

        for ( unsigned i = 0; i < NUMBER_OF_EVENTS; ++i )
        {
            if ( i != eventType )
            {
                result.append( Event( i ) );
            }
        }

        return result;
    }

    static String eventToString( Event &event )
    {
        switch ( event.type() )
        {
        case 0:
            return "HTTP_DATA_RECEIVED";
            break;
        case 1:
            return "HTTP_DONE_HANDLING_REQUEST";
            break;
        case 2:
            return "HTTP_END_SESSION";
            break;
        case 3:
            return "HTTP_GENERATE_ERROR";
            break;
        case 4:
            return "HTTP_GENERATE_FOUND";
            break;
        case 5:
            return "HTTP_GET_REQUEST";
            break;
        case 6:
            return "TCP_DATA_RECEIVED";
            break;
        case 7:
            return "TCP_DONE_HADNLING_ACK";
            break;
        case 8:
            return "TCP_DONE_HANDLING_DATA";
            break;
        case 9:
            return "TCP_FIN_REQUEST";
            break;
        case 10:
            return "TCP_PUSH_REQUEST";
            break;
        case 11:
            return "TCP_SEND_ACK";
            break;
        case 12:
            return "TCP_SEND_DATA";
            break;
        case 13:
            return "TCP_SEND_PUSH";
            break;
        case 14:
            return "TCP_SYN_REQUEST";
            break;
        case 15:
            return "TCP_ANNOUNCE_TARGET_PORT";
            break;
        case 16:
            return "SYN_HANDLER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 17:
            return "SEQUENCE_NUMBER_FOR_SYN_HANDLER";
            break;
        case 18:
            return "FIN_HANDLER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 19:
            return "SEQUENCE_NUMBER_FOR_FIN_HANDLER";
            break;
        case 20:
            return "ACK_SENDER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 21:
            return "SEQUENCE_NUMBER_FOR_ACK_SENDER";
            break;
        case 22:
            return "PUSH_SENDER_REQUEST_SEQUENCE_NUMBER";
            break;
        case 23:
            return "SEQUENCE_NUMBER_FOR_PUSH_SENDER";
            break;
        case 24:
            return "DATA_FROM_TCP_TO_HTTP";
            break;
        case 25:
            return "DATA_FROM_HTTP_TO_TCP";
            break;
        case 26:
            return "NUMBER_OF_EVENTS";
            break;
        default:
            break;
        }

        return "\t\tUNKNOWN";
    }
};

#endif // __IEvent_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
