#ifndef __IpHeader_h__
#define __IpHeader_h__

#include "HeapData.h"
#include <arpa/inet.h>

struct IpHeader
{
public:
    enum {
        IHL_BIT_MASK = 0x0f,

        BYTES_IN_LONG = 4,
    };

    struct MandatoryPart
    {
        unsigned char _versionAndIhl;
        unsigned char _dscpAndEcn;
        unsigned short _totalLength;
        unsigned short _identification;
        unsigned char _flags;
        unsigned char _offest;
        unsigned char _timeToLive;
        unsigned char _protocol;
        unsigned short _headerChecksum;
        unsigned _sourceIp;
        unsigned _destinationIp;
    } __attribute((packed));

    MandatoryPart _mandatoryPart;
    HeapData _options;

    void unserialize( const ConstSimpleData &serialized )
    {
        memcpy( &_mandatoryPart, serialized.data(), sizeof(MandatoryPart) );

        unsigned internetHeaderLength = ( _mandatoryPart._versionAndIhl & IHL_BIT_MASK ) * BYTES_IN_LONG;
        _options = ConstSimpleData( serialized.asChar() + sizeof(_mandatoryPart), internetHeaderLength - sizeof(_mandatoryPart) );

        _mandatoryPart._totalLength = ntohs( _mandatoryPart._totalLength );
        _mandatoryPart._identification = ntohs( _mandatoryPart._identification );
        _mandatoryPart._headerChecksum = ntohs( _mandatoryPart._headerChecksum );

        _mandatoryPart._sourceIp = ntohl( _mandatoryPart._sourceIp );
        _mandatoryPart._destinationIp = ntohl( _mandatoryPart._destinationIp );
    }

    unsigned size() const
    {
        return sizeof(_mandatoryPart) + _options.size();
    }
};

#endif // __IpHeader_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
