#ifndef __RawSocketManager_h__
#define __RawSocketManager_h__

#include "BThread.h"
#include "EngineConfiguration.h"
#include "Event.h"
#include "IEvent.h"
#include "IpHeader.h"
#include "RawSocket.h"
#include "Selector.h"
#include "ServerConfiguration.h"

class RawSocketManager : public BThread, public IRawSocket::IRawDataHandler
{
public:
    RawSocketManager() : _socket( _selector, ServerConfiguration::SERVER_IP, *this ), _busy( false )
    {
    }

    void entryPoint()
    {
        _socket.activate();

        while ( true )
        {
            select();

            while ( busyWithLastPacket() )
            {
                listenToOtherThreads();
            }
        }
    }

    void handleData( const ConstSimpleData &data, struct sockaddr_in & )
    {
        _busy = true;
        _indexInData = 0;

        while ( _indexInData < data.size() )
        {
            handleSinglePacket( data );
        }
    }

private:
    Selector _selector;
    RawSocket _socket;
    bool _busy;
    unsigned _indexInData;

    bool busyWithLastPacket() const
    {
        return _busy;
    }

    void select()
    {
        _selector.select();
    }

    void listenToOtherThreads()
    {
        waitForSendOrDone();

        switch ( lastEvent().type() )
        {
        case IEvent::TCP_DONE_HANDLING_DATA:
            doneHandlingIpPacket();
            break;
        case IEvent::TCP_SEND_DATA:
            sendIpData();
            break;
        default:
            break;
        }
    }

    void waitForSendOrDone()
    {
        clearEventVectors();
        _watched.append( IEvent::TCP_DONE_HANDLING_DATA );
        _watched.append( IEvent::TCP_SEND_DATA );
        _blocked.append( IEvent::TCP_DATA_RECEIVED );
        bSync( _requested, _watched, _blocked );
    }

    void doneHandlingIpPacket()
    {
        _busy = false;
    }

    void sendIpData()
    {
        _socket.send( lastEvent().payload(), ServerConfiguration::SERVER_IP );
    }

    // ConstSimpleData stripIpHeader( const ConstSimpleData &data )
    // {
    //     IpHeader ipHeader;
    //     ipHeader.unserialize( data );
    //     return ConstSimpleData( data.asChar() + ipHeader.size(), data.size() - ipHeader.size() );
    // }

    void handleSinglePacket( const ConstSimpleData &data )
    {
        IpHeader ipHeader;
        ipHeader.unserialize( ConstSimpleData( data.asChar() + _indexInData, data.size() - _indexInData ) );

        clearEventVectors();
        _requested.append( Event( IEvent::TCP_DATA_RECEIVED, ConstSimpleData( data.asChar() + ipHeader.size(), ipHeader._mandatoryPart._totalLength - ipHeader.size() ) ) );
        bSync( _requested, _watched, _blocked );

        _indexInData += ipHeader._mandatoryPart._totalLength;
    }
};

#endif // __RawSocketManager_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
