#ifndef __TcpDispatcherSimulator_h__
#define __TcpDispatcherSimulator_h__

class TcpDispatcherSimulator : public BThread
{
public:
    void entryPoint()
    {
        while ( true )
        {
            waitForRequests();
            classifyRequest();
        }
    }

private:
    void waitForRequests()
    {
        clearEventVectors();
        _watched.append( IEvent::TCP_DATA_RECEIVED );
        _blocked = allRelevantEventsExcept( IEvent::TCP_DATA_RECEIVED );
        bSync( _requested, _watched, _blocked, "Wait" );
    }

    void classifyRequest()
    {
        clearEventVectors();

        _requested.append( IEvent::TCP_DONE_HANDLING_DATA );
        _requested.append( IEvent::TCP_SYN_REQUEST );
        _requested.append( IEvent::TCP_FIN_REQUEST );
        _requested.append( IEvent::TCP_PUSH_REQUEST );

        _blocked.append( IEvent::TCP_DATA_RECEIVED );

        AtomicPropositions dispatching;
        dispatching.add( "TcpDispatching" );

        bSync( _requested, _watched, _blocked, dispatching, "Dispatch" );
    }

    Vector<Event> allRelevantEventsExcept( Event event )
    {
        Vector<Event> result;

        if ( event != IEvent::TCP_SYN_REQUEST ) result.append( IEvent::TCP_SYN_REQUEST );
        if ( event != IEvent::TCP_FIN_REQUEST ) result.append( IEvent::TCP_FIN_REQUEST );
        if ( event != IEvent::TCP_PUSH_REQUEST ) result.append( IEvent::TCP_PUSH_REQUEST );
        if ( event != IEvent::TCP_DATA_RECEIVED ) result.append( IEvent::TCP_DATA_RECEIVED );

        return result;
    }
};

#endif // __TcpDispatcherSimulator_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
