#ifndef __TcpPushHandler_h__
#define __TcpPushHandler_h__

#include "BThread.h"
#include "IEvent.h"

class TcpPushHandler : public BThread
{
public:
    void entryPoint()
    {
        while ( true )
        {
            waitForTcpPushRequest();

            storePushRequest();
            ackPacket();
            waitForAckToBeSent();

            sendDataToApplicationLayer();
            waitForApplicationLayerToFinish();

            doneHandlingIpPacket();
        }
    }

private:
    unsigned _sequenceNumber;
    TcpSegment _pushRequest;

    void storePushRequest()
    {
        _pushRequest.unserialize( lastEvent().payload() );
    }

    void waitForTcpPushRequest()
    {
        clearEventVectors();

        _watched.append( IEvent::TCP_PUSH_REQUEST ) ;
        _blocked = allRelevantEventsExcept( IEvent::TCP_PUSH_REQUEST );
        bSync( _requested, _watched, _blocked, "Wait" );
    }

    void ackPacket()
    {
        clearEventVectors();

        _requested.append( Event( IEvent::TCP_SEND_ACK, lastEvent().payload() ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_SEND_ACK );
        bSync( _requested, _watched, _blocked, "SendAck" );
    }

    void sendDataToApplicationLayer()
    {
        clearEventVectors();

        _requested.append( Event( IEvent::DATA_FROM_TCP_TO_HTTP, _pushRequest._payload ) );
        _blocked = allRelevantEventsExcept( IEvent::DATA_FROM_TCP_TO_HTTP );
        bSync( _requested, _watched, _blocked, "SendData" );
    }

    void waitForAckToBeSent()
    {
        clearEventVectors();

        _watched.append( Event( IEvent::TCP_DONE_HADNLING_ACK ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_DONE_HADNLING_ACK );
        bSync( _requested, _watched, _blocked, "DoneWithAck" );
    }

    void waitForApplicationLayerToFinish()
    {
        clearEventVectors();

        _watched.append( Event( IEvent::HTTP_DONE_HANDLING_REQUEST ) );
        _blocked = allRelevantEventsExcept( IEvent::HTTP_DONE_HANDLING_REQUEST );
        bSync( _requested, _watched, _blocked, "DoneWithData" );
    }

    void doneHandlingIpPacket()
    {
        clearEventVectors();
        _requested.append( Event( IEvent::TCP_DONE_HANDLING_DATA ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_DONE_HANDLING_DATA );
        bSync( _requested, _watched, _blocked, "Done" );
    }

    Vector<Event> allRelevantEventsExcept( Event event )
    {
        Vector<Event> result;

        if ( event != IEvent::TCP_PUSH_REQUEST ) result.append( IEvent::TCP_PUSH_REQUEST );
        if ( event != IEvent::DATA_FROM_TCP_TO_HTTP ) result.append( IEvent::DATA_FROM_TCP_TO_HTTP );
        if ( event != IEvent::HTTP_DONE_HANDLING_REQUEST ) result.append( IEvent::HTTP_DONE_HANDLING_REQUEST );

        return result;
    }
};

#endif // __TcpPushHandler_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
