#ifndef __SequenceMasterSpanner_h__
#define __SequenceMasterSpanner_h__

#include "SequenceMaster.h"
#include "StateGraphSpanner.h"

class SequenceMasterToken : public StateGraphSpanner::ThreadToken
{
public:
    BThread *freshCopy()
    {
        return new SequenceMaster;
    }
};

class SequenceMasterSpanner
{
public:
    void eventsForSequenceMaster( Vector<Event> &events )
    {
        events.clear();
        events = IEvent::allEvents();

        events.eraseByValue( IEvent::SYN_HANDLER_REQUEST_SEQUENCE_NUMBER );
        events.eraseByValue( IEvent::FIN_HANDLER_REQUEST_SEQUENCE_NUMBER );
        events.eraseByValue( IEvent::ACK_SENDER_REQUEST_SEQUENCE_NUMBER );
        events.eraseByValue( IEvent::PUSH_SENDER_REQUEST_SEQUENCE_NUMBER );

        SequenceNumberPayload sequenceNumber( 1, 1 );
        events.append( Event( IEvent::SYN_HANDLER_REQUEST_SEQUENCE_NUMBER, ConstSimpleData( &sequenceNumber, sizeof(sequenceNumber) ) ) );
        events.append( Event( IEvent::FIN_HANDLER_REQUEST_SEQUENCE_NUMBER, ConstSimpleData( &sequenceNumber, sizeof(sequenceNumber) ) ) );
        events.append( Event( IEvent::ACK_SENDER_REQUEST_SEQUENCE_NUMBER, ConstSimpleData( &sequenceNumber, sizeof(sequenceNumber) ) ) );
        events.append( Event( IEvent::PUSH_SENDER_REQUEST_SEQUENCE_NUMBER, ConstSimpleData( &sequenceNumber, sizeof(sequenceNumber) ) ) );
    }

    void span( ThreadStateGraph &stateGraph )
    {
        eventsForSequenceMaster( stateGraph._events );

        SequenceMasterToken token;
        StateGraphSpanner().span( token, stateGraph );

        stateGraph._threadName = "sequenceMaster";
    }
};

#endif // __SequenceMasterSpanner_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
